import "./chunk-QRPWKJ4C.js";

// src/typespec/core/packages/compiler/dist/manifest.js
var manifest_default = {
  "version": "1.4.0",
  "commit": "2375d080db0723d1ffdaa409cd3b305b64708dcc"
};
export {
  manifest_default as default
};
