import "../chunk-EPAU7SBM.js";
import {
  getMergePatchProperties,
  getMergePatchPropertyOverrides,
  getMergePatchPropertySource,
  getMergePatchSource,
  isMergePatch,
  setMergePatchPropertyOverrides,
  setMergePatchPropertySource,
  setMergePatchSource
} from "../chunk-7YEX5UX7.js";
export {
  getMergePatchProperties,
  getMergePatchPropertyOverrides,
  getMergePatchPropertySource,
  getMergePatchSource,
  isMergePatch,
  setMergePatchPropertyOverrides,
  setMergePatchPropertySource,
  setMergePatchSource
};
